package com.huimv.business.farm.controller;

import com.huimv.business.farm.service.IFarmDevice;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.io.UnsupportedEncodingException;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@CrossOrigin
@RestController
@Slf4j
@RequestMapping(value = "/farm/device")
public class FarmDeviceController {
    @Autowired
    private IFarmDevice iFarmDevice;

    /**
     * @Method      : newDevice
     * @Description : 新建设备信息
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/16       
     * @Time        : 14:42
     */
    @RequestMapping(value = "/newDevice",method = RequestMethod.POST)
    public Result newDevice(@RequestBody String deviceData) throws UnsupportedEncodingException {
        log.info("接收前端参数>>"+deviceData);
        //新建设备信息
        return iFarmDevice.newDevice(deviceData);
    }

    /**
     * @Method      : editDevice
     * @Description : 编辑设备信息
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/16       
     * @Time        : 14:43
     */
    @RequestMapping(value = "/editDevice",method = RequestMethod.POST)
    public Result editDevice(@RequestBody String deviceData) throws UnsupportedEncodingException {
        log.info("接收前端参数>>"+deviceData);
        //编辑设备信息
        return iFarmDevice.editDevice(deviceData);
    }

    /**
     * @Method      : removeDevice
     * @Description : 删除设备信息
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/16       
     * @Time        : 14:43
     */
    @RequestMapping(value = "/removeDevice",method = RequestMethod.POST)
    public Result removeDevice(@RequestBody String deviceData){
        log.info("接收前端参数>>"+deviceData);
        //删除设备信息
        return iFarmDevice.removeDevice(deviceData);
    }

    /**
     * @Method      : listDevice
     * @Description : 展示设备列表
     * @Params      : [deviceData]
     * @Return      : com.huimv.common.utils.Result
     *
     * @Author      : ZhuoNing
     * @Date        : 2021/12/16
     * @Time        : 14:43
     */
    @RequestMapping(value = "/listDevice",method = RequestMethod.POST)
    public Result listDevice(@RequestBody String deviceData){
        log.info("接收前端参数>>"+deviceData);
        //列表展示设备信息
        return iFarmDevice.listDevice(deviceData);
    }

}
