package com.huimv.business.server.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.huimv.business.server.dao.entity.FarmDeviceEntity;
import com.huimv.business.server.dao.repo.FarmDeviceRepo;
import com.huimv.business.server.service.IDevice;
import com.huimv.business.utils.TextUtil;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.UnsupportedEncodingException;
import java.sql.Date;
import java.sql.Timestamp;
import java.util.List;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class DeviceImpl implements IDevice {
    @Autowired
    private FarmDeviceRepo deviceRepo;
    @Autowired
    private TextUtil textUtil;

    @Override
    public Result syncAddFarmDevice(String deviceData) throws UnsupportedEncodingException {
        //转义符号处理
        JSONObject deviceDataJo = JSON.parseObject(textUtil.translateSymbol(deviceData));
        FarmDeviceEntity deviceEntity = new FarmDeviceEntity();
        deviceEntity.setDataId(deviceDataJo.getLong("id"));
        deviceEntity.setDeviceCode(deviceDataJo.getString("deviceCode"));
        deviceEntity.setDeviceName(deviceDataJo.getString("deviceName"));
        deviceEntity.setDeviceType(deviceDataJo.getString("deviceType"));
        deviceEntity.setDownTime(new Timestamp(new  java.util.Date().getTime()));
        deviceEntity.setFactory(deviceDataJo.getString("factory"));
        deviceEntity.setFarmId(deviceDataJo.getInteger("farmId"));
        deviceEntity.setLastTime(new Timestamp(new  java.util.Date().getTime()));
        deviceEntity.setLockinState(2);
        deviceEntity.setMainParams(deviceDataJo.getString("mainParams"));
        deviceEntity.setRecord(deviceDataJo.getString("record"));
        deviceEntity.setState(deviceDataJo.getInteger("state"));
        deviceEntity.setWorker(deviceDataJo.getString("worker"));
        FarmDeviceEntity addEntity = deviceRepo.saveAndFlush(deviceEntity);
        log.info("同步添加牧场端设备数据>>"+addEntity);
        return new Result(ResultCode.SUCCESS,addEntity);
    }

    //转义处理
//    public String _translateSymbol(String data) throws UnsupportedEncodingException {
//        return textUtil.decode(data.replace("data=","").replace("%3D","="));
//    }

    @Override
    public Result syncEditFarmDevice(String deviceData) throws UnsupportedEncodingException {
        //转义符号处理
        JSONObject deviceDataJo = JSON.parseObject(textUtil.translateSymbol(deviceData));
        Integer dataId = deviceDataJo.getInteger("id");
        //查询目标设备数据
        List<Object[]> existList = deviceRepo.getDevice(dataId);
        if(existList.size()>0) {
            Object[] object = existList.get(0);
            Integer id = (Integer) object[0];
            //更新目标设备数据
            FarmDeviceEntity deviceEntity = new FarmDeviceEntity();
            deviceEntity.setId(id);
            deviceEntity.setDataId(deviceDataJo.getLong("id"));
            deviceEntity.setDeviceCode(deviceDataJo.getString("deviceCode"));
            deviceEntity.setDeviceName(deviceDataJo.getString("deviceName"));
            deviceEntity.setDeviceType(deviceDataJo.getString("deviceType"));
            deviceEntity.setDownTime(new Timestamp(new java.util.Date().getTime()));
            deviceEntity.setFactory(deviceDataJo.getString("factory"));
            deviceEntity.setFarmId(deviceDataJo.getInteger("farmId"));
            deviceEntity.setLastTime(new Timestamp(new java.util.Date().getTime()));
            deviceEntity.setLockinState(2);
            deviceEntity.setMainParams(deviceDataJo.getString("mainParams"));
            deviceEntity.setRecord(deviceDataJo.getString("record"));
            deviceEntity.setState(deviceDataJo.getInteger("state"));
            deviceEntity.setWorker(deviceDataJo.getString("worker"));
            FarmDeviceEntity editEntity = deviceRepo.saveAndFlush(deviceEntity);
            log.info("同步编辑牧场端设备数据>>" + editEntity);
            return new Result(ResultCode.SUCCESS,editEntity);
        }else{
            log.info("错误：牧场端不存在data_id为"+dataId+"的记录。");
            return new Result(ResultCode.SUCCESS);
        }
    }

    @Override
    public Result syncRemoveFarmDevice(String deviceData) throws UnsupportedEncodingException {
        //转义符号处理
        Integer dataId = Integer.parseInt(textUtil.translateSymbol(deviceData));
        //查询目标设备数据
        List<Object[]> existList = deviceRepo.getDevice(dataId);
        if(existList.size()>0) {
            Object[] object = existList.get(0);
            Integer id = (Integer) object[0];
            deviceRepo.deleteById(id);
            log.info("同步删除牧场端设备数据.");
            return new Result(ResultCode.SUCCESS);
        }else{
            String ERR_INFO="暂无需要删除的设备数据.";
            log.info(ERR_INFO);
            return new Result(10001,ERR_INFO,false);
        }
    }
}
