package com.huimv.environ.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.huimv.environ.dao.entity.EnvHourEntity;
import com.huimv.environ.dao.entity.EnvMinuteEntity;
import com.huimv.environ.dao.entity.SysThresholdEntity;
import com.huimv.environ.dao.repo.EnvHourRepo;
import com.huimv.environ.dao.repo.EnvMinuteRepo;
import com.huimv.environ.service.IEnviron;
import com.huimv.environ.service.IThreshold;
import com.huimv.environ.utils.Const;
import com.huimv.environ.utils.DateUtil;
import com.huimv.environ.utils.HttpTemplete;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Service
@Slf4j
public class EnvironImpl implements IEnviron {
    @Autowired
    private HttpTemplete httpTemplete;
    @Autowired
    private EnvHourRepo envHourRepo;
    @Autowired
    private EnvMinuteRepo envMinuteRepo;
    //数据中心地址
    @Value("${dataCenter.ipAddr}")
    private String ipAddr ;
    //数据中心地址
    @Value("${dataCenter.port}")
    private String port ;
    //牧场ID
    @Value("${local.farmID}")
    private Integer farmID;
    //小时数据服务
    @Value("${dataCenter.service.hourEnviron}")
    private String hourEnvironService;
    //实时数据服务
    @Value("${dataCenter.service.realTimeEnviron}")
    private String realTimeEnvironService;
    //环境警报
    @Value("${dataCenter.service.alarm}")
    private String alarmService;
    @Autowired
    private IThreshold Threshold;
    @Autowired
    private DateUtil dateUtil;

    //本地测试地址
//    String testLocalHttpIp = "http://192.168.1.49:10091";

    @Override
    public void sendEnviron() throws IOException {
//        String url = ipAddr + port + "/cloud/environ/putHourEnviron";
        String url = ipAddr + port + hourEnvironService;
        log.info("推送地址："+url);
        //
        Map<String,String> paramsMap = new HashMap<String,String>();
        paramsMap.put("data", getEnvironData().toJSONString());
        //
        Map<String,Integer> timeoutMap = new HashMap<String,Integer>();
        timeoutMap.put("connectTimeout", 5000);
        timeoutMap.put("requestTimeout", 5000);
        timeoutMap.put("socketTimeout", 5000);
        // 用Post方法推送接口数据
        httpTemplete.doPost(url,paramsMap,timeoutMap);
//        System.out.println("测试结果="+outJo);
    }

    //发送小时环保数据
    @Override
    public void sendHourEnviron(String data) {
//        String url = ipAddr + port + "/cloud/environ/putHourEnviron";
        String url = ipAddr + port + hourEnvironService;
        log.info("推送数据地址："+url);
        //
        Map<String,String> paramsMap = new HashMap<String,String>();
        paramsMap.put("data", data);
        //
        Map<String,Integer> timeoutMap = new HashMap<String,Integer>();
        timeoutMap.put("connectTimeout", 5000);
        timeoutMap.put("requestTimeout", 5000);
        timeoutMap.put("socketTimeout", 5000);
        try{
            // 用Post方法推送接口数据
            httpTemplete.doPost(url,paramsMap,timeoutMap);
        } catch (IOException e) {
            System.out.println("###错误信息："+e.getMessage());
        }
    }

    //发送简单数据
    public void _doPostSimple(String service,String data){
        String url = ipAddr + port + service;
        log.info("推送数据地址："+url);
        //
        Map<String,String> paramsMap = new HashMap<String,String>();
        paramsMap.put("data", data);
        //
        Map<String,Integer> timeoutMap = new HashMap<String,Integer>();
        timeoutMap.put("connectTimeout", 5000);
        timeoutMap.put("requestTimeout", 5000);
        timeoutMap.put("socketTimeout", 5000);
        try{
            // 用Post方法推送接口数据
            httpTemplete.doPost(url,paramsMap,timeoutMap);
        } catch (IOException e) {
            System.out.println("###错误信息："+e.getMessage());
        }
    }

    //发送实时数据
    @Override
    public void sendRealTimeEnviron(String data) {
//        String url = ipAddr + port + "/cloud/environ/putRealTimeEnviron";
        String url = ipAddr + port + realTimeEnvironService;
        log.info("推送数据地址："+url);
        //
        Map<String,String> paramsMap = new HashMap<String,String>();
        paramsMap.put("data", data);
        //
        Map<String,Integer> timeoutMap = new HashMap<String,Integer>();
        timeoutMap.put("connectTimeout", 5000);
        timeoutMap.put("requestTimeout", 5000);
        timeoutMap.put("socketTimeout", 5000);
        try{
            // 用Post方法推送接口数据
            httpTemplete.doPost(url,paramsMap,timeoutMap);
        } catch (IOException e) {
//            e.printStackTrace();
            System.out.println("###错误信息："+e.getMessage());
        }
    }

    @Override
    public JSONArray getEnvironData(){
//        EnvMinuteEntity envMinuteEntity = new EnvMinuteEntity();
//        EnvHourEntity envHourEntity = new EnvHourEntity();
        List<EnvMinuteEntity> envMinuteEntityList = envMinuteRepo.findAll();
        JSONArray  envMinuteJa = (JSONArray) JSONArray.toJSON(envMinuteEntityList);
        log.info("本次上传记录数据："+envMinuteJa.size());
        return envMinuteJa;
    }

    /**
     * @Method      : handleEnvironWarningInfo
     * @Description : 处理分钟环保数据进行发送警报信息
     * @Params      : [data]
     * @Return      : void
     * 
     * @Author      : ZhuoNing
     * @Date        : 2021/12/3       
     * @Time        : 22:12
     */
    @Override
    public void handleEnvironWarningInfo(String data) throws ParseException {
        JSONObject environJo = JSON.parseObject(data);
        //读取阈值记录
        String thresholdData = Threshold.getThreshold(farmID);
        if(thresholdData == null){
            log.info("阈值记录为空.");
        }else{
            JSONObject thresholdJo = JSON.parseObject(thresholdData);
            //环保氨氮数据>=阈值氨氮数据
            if(new BigDecimal(environJo.getString("adPj")).compareTo(new BigDecimal(thresholdJo.getString("ammonia"))) > -1){
                String alarmInfo = "当前氨氮数据为"+environJo.getString("adPj")+"高于或等于阈值"+thresholdJo.getString("ammonia")+",特此警告.";
                //向数据中心发送警报
                _sendAlarmToCenter(alarmInfo,Const.TYPE_ALARM_AD);
            }

            //环保COD数据>=阈值COD数据
            if(new BigDecimal(environJo.getString("hxxylPj")).compareTo(new BigDecimal(thresholdJo.getString("cOD"))) > -1){
                String alarmInfo = "当前化学需氧量数据为"+environJo.getString("hxxylPj")+"高于或等于阈值"+thresholdJo.getString("cOD")+",特此警告.";
                //向数据中心发送警报
                _sendAlarmToCenter(alarmInfo,Const.TYPE_ALARM_COD);
            }
            //环保PH数据>=阈值PH数据
            if(new BigDecimal(environJo.getString("phPj")).compareTo(new BigDecimal(thresholdJo.getString("ph"))) > -1){
                String alarmInfo = "当前PH值数据为"+environJo.getString("phPj")+"高于或等于阈值"+thresholdJo.getString("ph")+",特此警告.";
                //向数据中心发送警报
                _sendAlarmToCenter(alarmInfo,Const.TYPE_ALARM_PH);
            }
            //环保污水浓度数据>=阈值污水浓度数据
            if(new BigDecimal(environJo.getString("wsPj")).compareTo(new BigDecimal(thresholdJo.getString("flow"))) > -1){
                String alarmInfo = "当前污水浓度数据为"+environJo.getString("wsPj")+"高于或等于阈值"+thresholdJo.getString("flow")+",特此警告.";
                //向数据中心发送警报
                _sendAlarmToCenter(alarmInfo,Const.TYPE_ALARM_WS);
            }
        }
    }

    //向数据中心发送警报
    public void _sendAlarmToCenter(String alarmInfo,Integer alarmType) throws ParseException {
        log.info("警报内容>>"+alarmInfo);
        String url = ipAddr + port + alarmService;
        log.info("推送警报地址>>"+url);
        // farmID,alarmInfo
        JSONObject alarmJo = new JSONObject();
        alarmJo.put("farmId",farmID);
        alarmJo.put("info",alarmInfo);
        alarmJo.put("level", Const.INFO_FIRST_LEVEL);
        alarmJo.put("levelName",Const.INFO_FIRST_LEVEL_ALARM);
        alarmJo.put("alarmType",alarmType);
        alarmJo.put("warningName",Const.INFO_ALARM_TYPE_NAME);
        alarmJo.put("alarmTime",dateUtil.getTodayText());
        //发送警报
        _doPostSimple(alarmService,alarmJo.toJSONString());
    }
}
